@@ -1,6 +1,7 @@
 package UR;
 
-# The UR module is itself a "UR::Namespace", besides being the root 
+
+# The UR module is itself a "UR::Namespace", besides being the root
 # module which bootstraps the system.  The class definition itself
 # is made at the bottom of the file.
 
@@ -8,20 +9,14 @@ use strict;
 use warnings FATAL => 'all';
 
 # Set the version at compile time, since some other modules borrow it.
-our $VERSION = "0.41"; # UR $VERSION
-
-BEGIN {
-    # unless otherwise specified, begin uncaching at 1 million objects 
-    #$ENV{'UR_CONTEXT_CACHE_SIZE_HIGHWATER'} ||= 1_000_000;
-    #$ENV{'UR_CONTEXT_CACHE_SIZE_LOWWATER'} ||= 1_000;
-}
+our $VERSION = "0.43"; # UR $VERSION
 
 # Ensure we get detailed errors while starting up.
 # This is disabled at the bottom of the module.
 use Carp;
 $SIG{__DIE__} = \&Carp::confess;
 
-# Ensure that, if the application changes directory, we do not 
+# Ensure that, if the application changes directory, we do not
 # change where we load modules while running.
 use Cwd;
 my @PERL5LIB = ($ENV{PERL5LIB} ? split(':', $ENV{PERL5LIB}) : ());
@@ -48,9 +43,9 @@ for my $e (keys %ENV) {
         my $path = __FILE__;
         $path =~ s/.pm$//;
         my @files = glob($path . '/Env/*');
-        my @vars = map { /UR\/Env\/(.*).pm/; $1 } @files; 
+        my @vars = map { /UR\/Env\/(.*).pm/; $1 } @files;
         print STDERR "Environment variable $e set to $ENV{$e} but there were errors using UR::Env::$e:\n"
-        . "Available variables:\n\t" 
+        . "Available variables:\n\t"
         . join("\n\t",@vars)
         . "\n";
         exit 1;
@@ -99,10 +94,10 @@ use Date::Format;
 
 #
 # Because UR modules execute code when compiling to define their classes,
-# and require each other for that code to execute, there are bootstrapping 
+# and require each other for that code to execute, there are bootstrapping
 # problems.
 #
-# Everything which is part of the core framework "requires" UR 
+# Everything which is part of the core framework "requires" UR
 # which, of course, executes AFTER it has compiled its SUBS,
 # but BEFORE it defines its class.
 #
@@ -122,7 +117,7 @@ require UR::ModuleConfig;   # used by ::Time, and also ::Lock ::Daemon
 require UR::Object::Iterator;
 require UR::DeletedRef;
 
-require UR::Object;         
+require UR::Object;
 require UR::Object::Type;
 
 require UR::Object::Ghost;
@@ -131,12 +126,12 @@ require UR::Object::Property;
 require UR::Observer;
 
 require UR::BoolExpr::Util;
-require UR::BoolExpr;                                  # has meta 
+require UR::BoolExpr;                                  # has meta
 require UR::BoolExpr::Template;                        # has meta
 require UR::BoolExpr::Template::PropertyComparison;    # has meta
 require UR::BoolExpr::Template::Composite;             # has meta
-require UR::BoolExpr::Template::And;                   # has meta    
-require UR::BoolExpr::Template::Or;                    # has meta  
+require UR::BoolExpr::Template::And;                   # has meta
+require UR::BoolExpr::Template::Or;                    # has meta
 
 require UR::Object::Index;
 
@@ -154,7 +149,8 @@ UR::Object::Type->define(
     composite_id_separator => "\t",
     id_by => [
         id  => { is => 'Scalar', doc => 'unique identifier' }
-    ]
+    ],
+    id_generator => '-urinternal',
 );
 
 UR::Object::Type->define(
@@ -196,27 +192,27 @@ UR::Object::Type->define(
     is_abstract => 1,
 
     has => [
-        class_name                       => { is => 'Text', len => 256, is_optional => 1, 
+        class_name                       => { is => 'Text', len => 256, is_optional => 1,
                                                 doc => 'the name for the class described' },
-        
-        properties                       => { 
+
+        properties                       => {
                                                 is_many => 1,
-                                                
+
                                                 # this is calculated instead of a regular relationship
-                                                # so we can do appropriate inheritance filtering.  
+                                                # so we can do appropriate inheritance filtering.
                                                 # We need an isa operator and its converse
                                                 # in order to be fully declarative internally here
                                                 calculate => 'shift->_properties(@_);',
-                                                
+
                                                 doc => 'property meta-objects for the class'
                                             },
         id_properties                    => { is_many => 1,
                                               calculate => q( grep { defined $_->is_id } shift->_properties(@_) ),
                                               doc => 'meta-objects for the ID properties of the class' },
 
-        doc                              => { is => 'Text', len => 1024, is_optional => 1, 
+        doc                              => { is => 'Text', len => 1024, is_optional => 1,
                                                 doc => 'a one-line description of the class/type' },
-      
+
         is_abstract                      => { is => 'Boolean', default_value => 0,
                                                 doc => 'abstract classes must be subclassified into a concreate class at create/load time' },
 
@@ -229,13 +225,13 @@ UR::Object::Type->define(
         is_singleton                     => { is => 'Boolean', default_value => 0,
                                                 doc => 'singleton classes have only one instance, or have each instance fall into a distinct subclass' },
 
-        namespace                        => { is => 'Text', len => 256, is_optional => 1, 
+        namespace                        => { is => 'Text', len => 256, is_optional => 1,
                                                 doc => 'the first "word" in the class name, which points to a UR::Namespace' },
-                                            
+
         schema_name                      => { is => 'Text', len => 256, is_optional => 1,
                                                 doc => 'an arbitrary grouping for classes for which instances share a common storage system' },
-        
-        data_source_id                   => { is => 'Text', len => 256, is_optional => 1, 
+
+        data_source_id                   => { is => 'Text', len => 256, is_optional => 1,
                                                 doc => 'for classes which persist beyond their current process, the identifier for their storage manager' },
 
         #data_source_meta                => { is => 'UR::DataSource', id_by => 'data_source_id', is_optional => 1,  },
@@ -243,16 +239,16 @@ UR::Object::Type->define(
         generated                        => { is => 'Boolean', is_transient => 1, default_value => 0,
                                                 doc => 'an internal flag set when the class meta has fabricated accessors and methods in the class namespace' },
 
-        meta_class_name                  => { is => 'Text', 
+        meta_class_name                  => { is => 'Text',
                                                 doc => 'even meta-classess have a meta-class' },
 
         composite_id_separator           => { is => 'Text', len => 2 , default_value => "\t", is_optional => 1,
-                                                doc => 'for classes whose objects have a multi-value "id", this overrides using a "\t" to compose/decompose' },        
-        
+                                                doc => 'for classes whose objects have a multi-value "id", this overrides using a "\t" to compose/decompose' },
+
         valid_signals                    => { is => 'ARRAY', is_optional => 1,
                                                 doc => 'List of non-standard signal names observers can bind to ' },
-        # details used by the managment of the "real" entity outside of the app (persistence) 
-        table_name                       => { is => 'Text', len => undef, is_optional => 1, 
+        # details used by the managment of the "real" entity outside of the app (persistence)
+        table_name                       => { is => 'Text', len => undef, is_optional => 1,
                                                 doc => 'for classes with a data source, this specifies the table or equivalent data structure which holds instances' },
 
         select_hint                       => { is => 'Text', len => 1024 , is_optional => 1,
@@ -260,28 +256,28 @@ UR::Object::Type->define(
 
         join_hint                        => { is => 'Text', len => 1024 , is_optional => 1,
                                                 doc => 'used to optimize access to underlying storage when this class is part of a join (database specific)' },
-                                            
+
         id_generator                     => { is => 'Text', len => 256, is_optional => 1,
                                                 doc => 'override the default choice for generating new object IDs' },
 
         # different ways of handling subclassing at object load time
         subclassify_by                      => { is => 'Text', len => 256, is_optional => 1,
                                                 doc => 'when set, the method specified will return the name of a specific subclass into which the object should go' },
-        
+
         subclass_description_preprocessor   => { is => 'MethodName', len => 255, is_optional => 1,
                                                 doc => 'a method which should pre-process the class description of sub-classes before construction' },
-        
-        sub_classification_method_name      => { is => 'Text', len => 256, is_optional => 1, 
+
+        sub_classification_method_name      => { is => 'Text', len => 256, is_optional => 1,
                                                 doc => 'like subclassify_by, but examines whole objects not a single property' },
 
         use_parallel_versions               => { is => 'Boolean', is_optional => 1, default_value => 0,
                                                 doc => 'inheriting from the is class will redirect to a ::V? module implemeting a specific version' },
-        
+
         # obsolete/internal
         type_name                               => { is => 'Text', len => 256, is_deprecated => 1, is_optional => 1 },
         er_role                                 => { is => 'Text', len => 256, is_optional => 1,  default_value => 'entity' },
         source                                  => { is => 'Text', len => 256 , default_value => 'data dictionary', is_optional => 1 }, # This is obsolete and should be removed later
-        sub_classification_meta_class_name      => { is => 'Text', len => 1024 , is_optional => 1, 
+        sub_classification_meta_class_name      => { is => 'Text', len => 1024 , is_optional => 1,
                                                     doc => 'obsolete' },
         first_sub_classification_method_name    => { is => 'Text', len => 256, is_optional => 1,
                                                     doc => 'cached value to handle a complex inheritance hierarchy with storage at some levels but not others' },
@@ -291,7 +287,7 @@ UR::Object::Type->define(
 
         # UR::Namespaces are singletons referenced through their name
         namespace_meta                  => { is => 'UR::Namespace', id_by => 'namespace' },
-        is                              => { is => 'ARRAY', is_mutable => 0, doc => 'List of the parent class names' },  
+        is                              => { is => 'ARRAY', is_mutable => 0, doc => 'List of the parent class names' },
 
         # linking to the direct parents, and the complete ancestry
         parent_class_metas              => { is => 'UR::Object::Type', id_by => 'is',
@@ -333,7 +329,7 @@ UR::Object::Type->define(
         direct_id_column_names           => { via => 'direct_id_property_metas', to => 'column_name', is_many => 1, where => [column_name => { operator => 'true'}] },
         ancestry_column_names            => { via => 'ancestry_class_metas', to => 'direct_column_names', is_many => 1 },
         ancestry_id_column_names         => { via => 'ancestry_class_metas', to => 'direct_id_column_names', is_many => 1 },
-        
+
         # Are these *columnless* properties actually necessary?  The user could just use direct_property_metas(column_name => undef)
         direct_columnless_property_metas => { is => 'UR::Object::Property', reverse_as => 'class_meta', where => [column_name => undef], is_many => 1 },
         direct_columnless_property_names => { via => 'direct_columnless_property_metas', to => 'property_name', is_many => 1 },
@@ -345,23 +341,23 @@ UR::Object::Type->define(
         all_id_column_names              => { via => 'all_class_metas', to => 'direct_id_column_names', is_many => 1 },
         all_columnless_property_metas    => { via => 'all_class_metas', to => 'direct_columnless_property_metas', is_many => 1 },
         all_columnless_property_names    => { via => 'all_class_metas', to => 'direct_columnless_property_names', is_many => 1 },
-    ],    
+    ],
 );
 
 UR::Object::Type->define(
     class_name => 'UR::Object::Property',
     id_properties => [
-        class_name                      => { is => 'Text', len => 256 },        
-        property_name                   => { is => 'Text', len => 256 },            
+        class_name                      => { is => 'Text', len => 256 },
+        property_name                   => { is => 'Text', len => 256 },
     ],
     has_optional => [
         property_type                   => { is => 'Text', len => 256 , is_optional => 1},
-        column_name                     => { is => 'Text', len => 256, is_optional => 1 },        
+        column_name                     => { is => 'Text', len => 256, is_optional => 1 },
         data_length                     => { is => 'Text', len => 32, is_optional => 1 },
         data_type                       => { is => 'Text', len => 256, is_optional => 1 },
         default_value                   => { is_optional => 1 },
         valid_values                    => { is => 'ARRAY', is_optional => 1, },
-        example_values                  => { is => 'ARRAY', is_optional => 1, },
+        example_values                  => { is => 'ARRAY', is_optional => 1, doc => 'example valid values; used to generate help text for Commands' },
         doc                             => { is => 'Text', len => 1000, is_optional => 1 },
         is_id                           => { is => 'Integer', default_value => undef, doc => 'denotes this is an ID property of the class, and ranks them' },
         is_optional                     => { is => 'Boolean' , default_value => 0},
@@ -375,7 +371,7 @@ UR::Object::Type->define(
         is_transactional                => { is => 'Boolean' , default_value => 1},  # STM works on these, and the object can possibly save outside the app
         is_abstract                     => { is => 'Boolean' , default_value => 0},
         is_concrete                     => { is => 'Boolean' , default_value => 1},
-        is_final                        => { is => 'Boolean' , default_value => 0},  
+        is_final                        => { is => 'Boolean' , default_value => 0},
         is_many                         => { is => 'Boolean' , default_value => 0},
         is_aggregate                    => { is => 'Boolean' , default_value => 0},
         is_deprecated                   => { is => 'Boolean', default_value => 0},
@@ -475,18 +471,18 @@ UR - rich declarative transactional objects
 
 =head1 VERSION
 
-This document describes UR version 0.41
+This document describes UR version 0.43
 
 =head1 SYNOPSIS
 
-    use UR; 
+    use UR;
 
     ## no database
 
     class Foo { is => 'Bar', has => [qw/prop1 prop2 prop3/] };
-    
+
     $o1 = Foo->create(prop1 => 111, prop2 => 222, prop3 => 333);
-    
+
     @o = Foo->get(prop2 => 222, prop1 => [101,111,121], 'prop3 between' => [200, 400]);
     # returns one object
 
@@ -494,7 +490,7 @@ This document describes UR version 0.41
 
     @o = Foo->get(prop2 => 222, prop1 => [101,111,121], 'prop3 between' => [200, 400]);
     # returns zero objects
-    
+
     @o = Foo->get(prop2 => 222, prop1 => [101,111,121], 'prop3 between' => [200, 400]);
     # returns one object again
 
@@ -508,8 +504,8 @@ This document describes UR version 0.41
         table_name => 'Animal'
     };
 
-    class Cat { 
-        is => 'Animal', 
+    class Cat {
+        is => 'Animal',
         has => [
             feet    => { is => 'Number', default_value => 4 },
             fur     => { is => 'Text', valid_values => [qw/fluffy scruffy/] },
@@ -523,14 +519,14 @@ This document describes UR version 0.41
     @cats = Cat->get(favorite_food => ['taters','sea bass']);
 
     $c = $cats[0];
-    
+
     print $c->feet,"\n";
-    
+
     $c->fur('scruffy');
 
     UR::Context->commit();
 
-  
+
 =head1 DESCRIPTION
 
 UR is a class framework and object/relational mapper for Perl.  It starts
@@ -538,7 +534,7 @@ with the familiar Perl meme of the blessed hash reference as the basis for
 object instances, and extends its capabilities with ORM (object-relational
 mapping) capabilities, object cache, in-memory transactions, more formal
 class definitions, metadata, documentation system, iterators, command line
-tools, etc. 
+tools, etc.
 
 UR can handle multiple column primary and foreign keys, SQL joins involving
 class inheritance and relationships, and does its best to avoid querying
@@ -553,35 +549,35 @@ in-memory transactions for both.
 
 =head2 Manuals
 
-L<ur> - command line interface 
+L<ur> - command line interface
 
 L<UR::Manual::Overview> - UR from Ten Thousand Feet
 
-L<UR::Manual::Tutorial> - Getting started with UR 
+L<UR::Manual::Tutorial> - Getting started with UR
 
 L<UR::Manual::Presentation> - Slides for a presentation on UR
 
-L<UR::Manual::Cookbook> - Recepies for getting stuff working 
+L<UR::Manual::Cookbook> - Recepies for getting stuff working
 
 L<UR::Manual::Metadata> - UR's metadata system
 
-L<UR::Object::Type::Initializer> - Defining classes 
+L<UR::Object::Type::Initializer> - Defining classes
 
 =head2 Basic Entities
 
 L<UR::Object> - Pretty much everything is-a UR::Object
 
-L<UR::Object::Type> - Metadata class for Classes 
+L<UR::Object::Type> - Metadata class for Classes
 
 L<UR::Object::Property> - Metadata class for Properties
 
-L<UR::Namespace> - Manage packages and classes 
+L<UR::Namespace> - Manage packages and classes
 
 L<UR::Context> - Software transactions and More!
 
 L<UR::DataSource> - How and where to get data
 
-=head1 QUICK TUTORIAL 
+=head1 QUICK TUTORIAL
 
 First create a Namespace class for your application, Music.pm:
 
@@ -598,7 +594,7 @@ Next, define a data source representing your database, Music/DataSource/DB1.pm
 
     package Music::DataSource::DB1;
     use Music;
-    
+
     class Music::DataSource::DB1 {
         is => ['UR::DataSource::MySQL', 'UR::Singleton'],
         has_constant => [
@@ -608,13 +604,13 @@ Next, define a data source representing your database, Music/DataSource/DB1.pm
             auth    => { value => 'mysqlpasswd' },
         ]
     };
-    
+
     or to get something going quickly, SQLite has smart defaults...
 
     class Music::DataSource::DB1 {
         is => ['UR::DataSource::SQLite', 'UR::Singleton'],
     };
-    
+
 
 Create a class to represent artists, who have many CDs, in Music/Artist.pm
 
@@ -623,7 +619,7 @@ Create a class to represent artists, who have many CDs, in Music/Artist.pm
 
     class Music::Artist {
         id_by => 'artist_id',
-        has => [ 
+        has => [
             name => { is => 'Text' },
             cds  => { is => 'Music::Cd', is_many => 1, reverse_as => 'artist' }
         ],
@@ -635,7 +631,7 @@ Create a class to represent CDs, in Music/Cd.pm
 
     package Music::Cd;
     use Music;
-            
+
     class Music::Cd {
         id_by => 'cd_id',
         has => [
@@ -655,7 +651,7 @@ If the database does not exist, you can run this to generate the tables and colu
   $ cd Music
   $ ur update schema
 
-If the database existed already, you could have done this to get it to write the last 2 classes for you: 
+If the database existed already, you could have done this to get it to write the last 2 classes for you:
 
   $ cd Music;
   $ ur update classes
@@ -663,8 +659,8 @@ If the database existed already, you could have done this to get it to write the
 Regardless, if the classes and database tables are present, you can then use these classes in your application code:
 
     # Using the namespace enables auto-loading of modules upon first attempt to call a method
-    use Music;  
-    
+    use Music;
+
     # This would get back all Artist objects:
     my @all_artists = Music::Artist->get();
 
@@ -677,7 +673,7 @@ Regardless, if the classes and database tables are present, you can then use the
 
     # Get all the CDs published in 2007 for the first artist
     my @cds_2007 = Music::Cd->get(year => 2007, artist => $first_artist);
-   
+
     # Use non-equality operators:
     my @some_cds = Music::Cd->get(
         'year between' => ['2004','2009']
@@ -687,13 +683,13 @@ Regardless, if the classes and database tables are present, you can then use the
     # the data in the database.  @some_cds will contain Music::Cd objects.
     # As a side effect, related Artist objects will be loaded into the cache
     @some_cds = Music::Cd->get(
-        year => '2007', 
-        'artist_name like' => 'Bob%' 
+        year => '2007',
+        'artist_name like' => 'Bob%'
     );
 
     # These values would be cached...
     my @artists_for_some_cds = map { $_->artist } @some_cds;
-    
+
     # This will use a join to prefetch Artist objects related to the
     # objects that match the filter
     my @other_cds = Music::Cd->get(
@@ -701,8 +697,8 @@ Regardless, if the classes and database tables are present, you can then use the
         -hints => ['artist']
     );
     my $other_artist_0 = $other_cds[0]->artist;  # already loaded so no query
-    
-    # create() instantiates a new object in the current "context", but does not save 
+
+    # create() instantiates a new object in the current "context", but does not save
     # it in the database.  It will autogenerate its own cd_id:
     my $new_cd = Music::Cd->create(
         title => 'Cool Album',
@@ -711,7 +707,7 @@ Regardless, if the classes and database tables are present, you can then use the
 
     # Assign it to an artist; fills in the artist_id field of $new_cd
     $first_artist->add_cd($new_cd);
-    
+
     # Save all changes in the current transaction back to the database(s)
     # which are behind the changed objects.
     UR::Context->current->commit;
@@ -719,8 +715,8 @@ Regardless, if the classes and database tables are present, you can then use the
 =head1 Environment Variables
 
 UR uses several environment variables to do things like run with
-database commits disabled, watching SQL queries run, examine query plans, 
-and control cache size, etc.  
+database commits disabled, watching SQL queries run, examine query plans,
+and control cache size, etc.
 
 These make development and debugging fast and easy.
 
@@ -748,20 +744,20 @@ XML::Simple
 =head1 AUTHORS
 
 UR was built by the software development team at The Genome Institute
-at Washington University School of Medicine (Richard K. Wilson, PI).  
+at Washington University School of Medicine (Richard K. Wilson, PI).
 
-Incarnations of it run laboratory automation and analysis systems 
+Incarnations of it run laboratory automation and analysis systems
 for high-throughput genomics.
- 
+
  Anthony Brummett   brummett@cpan.org
- Nathan Nutter                          
+ Nathan Nutter
  Josh McMichael
- Eric Clark                              
+ Eric Clark
  Ben Oberkfell
  Eddie Belter
  Feiyu Du
  Adam Dukes
- Brian Derickson    
+ Brian Derickson
  Craig Pohl
  Gabe Sanderson
  Todd Hepler
@@ -791,9 +787,9 @@ for high-throughput genomics.
  Travis Abbott
  Matthew Callaway
  James Eldred
- Scott Smith	    sakoht@cpan.org	    
+ Scott Smith	    sakoht@cpan.org
  David Dooling
- 
+
 =head1 LICENCE AND COPYRIGHT
 
 Copyright (C) 2002-2011 Washington University in St. Louis, MO.
