@@ -3,7 +3,7 @@ package UR::DataSource::Filesystem;
 use UR;
 use strict;
 use warnings;
-our $VERSION = "0.41"; # UR $VERSION;
+our $VERSION = "0.43"; # UR $VERSION;
 
 use File::Basename;
 use File::Path;
@@ -56,6 +56,9 @@ class UR::DataSource::Filesystem {
 
 sub can_savepoint { 0;}  # Doesn't support savepoints
 
+# Filesystem datasources don't have a "default_handle"
+sub create_default_handle { undef }
+
 sub _regex {
     my $self = shift;
 
