@@ -43,7 +43,7 @@ a formal UR class.
 require 5.006_000;
 use warnings;
 use strict;
-our $VERSION = "0.41"; # UR $VERSION;;
+our $VERSION = "0.43"; # UR $VERSION;;
 
 # set up module
 use Carp;
@@ -464,14 +464,18 @@ For example, for the "error" message type, these methods are created:
 =item error_message
 
     $obj->error_message("Something went wrong...");
+    $obj->error_message($format, @list);
     $msg = $obj->error_message();
 
-When called with one argument, it sends an error message to the object.  The 
-error_message_callback will be run, if one is registered, and the message will
-be printed to the terminal.  When called with no arguments, the last message
-sent will be returned.  If the message is C<undef> then no message is printed
-or queued, and the next time error_message is run as an accessor, it will
-return undef.
+When called with one or more arguments, it sends an error message to the
+object.  The error_message_callback will be run, if one is registered, and the
+message will be printed to the terminal.  When given a single argument, it will
+be passed through unmodified.  When given multiple arguments, error_message will
+assume the first is a format string and the remainder are parameters to
+sprintf.  When called with no arguments, the last message sent will be
+returned.  If the message is C<undef> then no message is printed or queued, and
+the next time error_message is run as an accessor, it will return
+undef.
 
 =item dump_error_messages
 
@@ -699,9 +703,16 @@ $create_subs_for_message_type = sub {
     my $logger_subref = Sub::Name::subname "${class}::${logger_subname}" => sub {
         my $self = shift;
 
-        foreach ( @_ ) {
-            my $msg = $_;
-            chomp($msg) if defined;
+        if (@_) {
+            my $msg = shift;
+
+            # if given multiple arguments, assume it's a format string
+            if(@_) {
+                $msg = _carp_sprintf($msg, @_);
+            }
+
+            defined($msg) && chomp($msg);
+
             # old-style callback registered with error_messages_callback
             if (my $code = $self->$check_callback()) {
                 if (ref $code) {
@@ -720,24 +731,25 @@ $create_subs_for_message_type = sub {
             $save_setting->($self, $logger_subname, $msg);
             # If the callback set $msg to undef with "$_[1] = undef", then they didn't want the message
             # processed further
-            next unless defined($msg);
+            if (defined $msg) {
 
-            if (my $fh = $self->$should_dump_messages()) {
-                $fh = $$default_fh unless (ref $fh);
+                if (my $fh = $self->$should_dump_messages()) {
+                    $fh = $$default_fh unless (ref $fh);
 
-                $fh->print($message_text_prefix . $msg . "\n");
+                    $fh->print($message_text_prefix . $msg . "\n");
+                }
+
+                if ($self->$should_queue_messages()) {
+                    my $a = $self->$messages_arrayref();
+                    push @$a, $msg;
+                }
+
+                my ($package, $file, $line, $subroutine) = caller;
+                $self->$message_package($package);
+                $self->$message_file($file);
+                $self->$message_line($line);
+                $self->$message_subroutine($subroutine);
             }
-
-            if ($self->$should_queue_messages()) {
-                my $a = $self->$messages_arrayref();
-                push @$a, $msg;
-            }
-
-            my ($package, $file, $line, $subroutine) = caller;
-            $self->$message_package($package);
-            $self->$message_file($file);
-            $self->$message_line($line);
-            $self->$message_subroutine($subroutine);
         }
 
         return $get_setting->($self, $logger_subname);
@@ -749,6 +761,30 @@ $create_subs_for_message_type = sub {
     });
 };
 
+sub _carp_sprintf {
+    my $format = shift;
+    my @list = @_;
+
+    # warnings weren't very helpful because they wouldn't tell you who passed
+    # in the "bad" format string
+    my $formatted_string;
+    my $warn_msg;
+    {
+        local $SIG{__WARN__} = sub {
+            my $msg = $_[0];
+            my ($filename, $line) = (caller)[1, 2];
+            my $short_msg = ($msg =~ /(.*) at $filename line $line./)[0];
+            $warn_msg = ($short_msg || $msg);
+        };
+        $formatted_string = sprintf($format, @list);
+    }
+    if ($warn_msg) {
+        Carp::carp($warn_msg);
+    }
+
+    return $formatted_string;
+}
+
 
 # at init time, make messaging subs for the initial message types
 $create_subs_for_message_type->(__PACKAGE__, $_) foreach @message_types;
