@@ -5,10 +5,10 @@ use strict;
 
 require UR;
 
-use Scalar::Util;
+use Scalar::Util qw(looks_like_number);
 
 our @ISA = ('UR::ModuleBase');
-our $VERSION = "0.41"; # UR $VERSION;;
+our $VERSION = "0.43"; # UR $VERSION;;
 
 # Base object API
 
@@ -57,7 +57,15 @@ sub __meta__  {
 # things like mapping operations easier and calculate_from metadata able
 # to include the object as function args to calculated properties
 sub __self__ {
-    return $_[0];
+    return $_[0] if @_ == 1;
+    my $self = shift;
+    my $bx = $self->class->define_boolexpr(@_);
+    if ($bx->evaluate($self)) {
+        return $self;
+    }
+    else {
+        return;
+    }
 }
 
 
@@ -159,6 +167,11 @@ sub __errors__ {
 
         my $value = $values[0];
 
+        # account for minus sign in dummy ID
+        if ($ENV{UR_USE_DUMMY_AUTOGENERATED_IDS} and $property_metadata->is_id and defined($value) and index($value, '-') == 0 and defined $data_length) {
+            $data_length++;
+        }
+
         if (! ($is_optional or defined($value))) {
             push @tags, UR::Object::Tag->create(
                             type => 'invalid',
@@ -174,51 +187,18 @@ sub __errors__ {
         # Check data type
         # TODO: delegate to the data type module for this
         $generic_data_type = '' unless (defined $generic_data_type);
-        if ($generic_data_type eq 'Float') {
-            $value =~ s/\s//g;
-            $value = $value + 0;
 
-            my $length =0;
-
-            if($value =~ /^(\+|\-)?([0-9]+)(\.([0-9]*))?[eE](\+|\-)?(\d+)$/){ #-- scientific notation
-                $length = length($2)-1 + $6 + (!$5 || $5 eq '+' ? 1 : 0);
-            }
-            elsif($value =~ /^(\+|\-)?([0-9]*)(\.([0-9]*))?$/) {
-                # If the data type is specified as a Float, but really contains an int, then
-                # $4 is undef causing a warning about "uninitialized value in concatenation",
-                # but otherwise works OK
-                no warnings 'uninitialized';
-                $length = length($2.$4);
-                --$length if $2 == 0 && $4;
+        if ($generic_data_type eq 'Float' || $generic_data_type eq 'Integer') {
+            if (looks_like_number($value)) {
+                $value = $value + 0;
             }
             else{
                 push @tags, UR::Object::Tag->create (
                     type => 'invalid',
                     properties => [$property_name],
-                    desc => 'Invalid decimal value.'
+                    desc => "Invalid $generic_data_type value."
                 );
             }
-            # Cleanup for size check below.
-            $value = '.' x $length;
-        }
-        elsif ($generic_data_type eq 'Integer') {
-            $value =~ s/\s//g;
-            if ($value =~ /\D/) {
-                #$DB::single = 1;
-                #print "$self $self->{id} $property_name @values\n";
-            }
-            $value = $value + 0;
-            if ($value !~ /^(\+|\-)?[0-9]*$/)
-            {
-                push @tags, UR::Object::Tag->create
-                (
-                    type => 'invalid',
-                    properties => [$property_name],
-                    desc => 'Invalid integer.'
-                );
-            }
-            # Cleanup for size check below.
-            $value =~ s/[\+\-]//g;
         }
         elsif ($generic_data_type eq 'DateTime') {
             # This check is currently disabled b/c of time format irrecularities
@@ -266,7 +246,8 @@ sub __errors__ {
                 }
             }
             unless ($valid) {
-                my $value_list = join(', ',@$constraints);
+                # undef is a valid value in the constraints list
+                my $value_list = join(', ',map { defined($_) ? $_ : '' } @$constraints);
                 push @tags,
                     UR::Object::Tag->create(
                         type => 'invalid',
@@ -308,7 +289,13 @@ sub __errors__ {
 #  mock objects
 
 sub define_boolexpr {
-    return UR::BoolExpr->resolve(@_);
+    if (ref($_[0])) {
+        my $class = ref(shift);
+        return UR::BoolExpr->resolve($class,@_);
+    }
+    else {
+        return UR::BoolExpr->resolve(@_);
+    }
 }
 
 sub define_set {
@@ -423,7 +410,7 @@ sub create_mock {
     for my $class_object ($subject_class_object,$subject_class_object->ancestry_class_metas) {
         for my $property ($class_object->direct_property_metas) {
             my $property_name = $property->property_name;
-            if ($property->is_delegated && !exists($params{$property_name})) {
+            if (($property->is_delegated || $property->is_optional) && !exists($params{$property_name})) {
                 next;
             }
             if ($property->is_mutable || $property->is_calculated || $property->is_delegated) {
@@ -484,15 +471,11 @@ sub __changes__ {
     # This is really only useful internally because the boundary of the object
     # is internal/subjective.
 
-    my ($self,$optional_property) = @_;
+    my $self = shift;
 
     # performance optimization
     return unless $self->{_change_count};
 
-    unless (wantarray) {
-        return $self->{_change_count};  # scalar context only cares if there are any changes or not
-    }
-
     my $meta = $self->__meta__;
     if (ref($meta) eq 'UR::DeletedRef') {
         print Data::Dumper::Dumper($self,$meta);
@@ -504,19 +487,43 @@ sub __changes__ {
 
     my $orig = $self->{db_saved_uncommitted} || $self->{db_committed};
 
+    my %prop_metas;
+    my $prop_is_changed = sub {
+        my $prop_name = shift;
+        my $property_meta = $prop_metas{$prop_name} ||= $meta->property_meta_for_name($prop_name);
+        no warnings 'uninitialized';
+        return ($orig->{$prop_name} ne $self->{$prop_name})
+                &&
+                ($self->can($prop_name) and ! UR::Object->can($prop_name))
+                &&
+                defined($property_meta)
+                &&
+               (! $property_meta->is_transient)
+            ;
+    };
+
+    unless (wantarray) {
+        # scalar context only cares if there are any changes or not
+        if (@_) {
+            foreach (@_) {
+                return 1 if $prop_is_changed->($_);
+            }
+            return '';
+        } else {
+            return ($self->{__defined} and $self->{_change_count} == 1)
+                    ? ''
+                    : $self->{_change_count};
+        }
+    }
+
     no warnings;
     my @changed;
     if ($orig) {
         my $class_name = $meta->class_name;
         @changed =
-            grep {
-                my $property_meta = $meta->property_meta_for_name($_);
-                ( ((!$property_meta) or $property_meta->is_transient) ? 0 : 1 );
-            }
-            grep { $self->can($_) and not UR::Object->can($_) }
-            grep { $orig->{$_} ne $self->{$_} }
+            grep { $prop_is_changed->($_) }
             grep { $_ }
-            keys %$orig;
+            @_ ? (@_) : keys(%$orig);
     }
     else {
         @changed = $meta->all_property_names
@@ -1273,8 +1280,11 @@ Called by all mutators to tell the current context about a state change.
 
   @tags = $obj->__changes__()
 
+  @tags = $obj->__changes__('prop1', 'prop2', ...)
+
 Return a list of changes present on the object _directly_.  This is really only
 useful internally because the boundary of the object is internal/subjective.
+Callers may also request only changes to particular properties.
 
 Changes to objects' properties are tracked by the system.  If an object has been
 changed since it was defined or loaded from its external data source, then changed()
